@@ -42,7 +42,13 @@ django.jQuery(function ($) {
         $floorplan = $floorplanRow.find('select').eq(0),
         $floorplanImage = $('.indoor.coords .field-image input'),
         $floorplanMap = $('.indoor.coords .floorplan-widget'),
-        isNew = true;
+        isNew = true,
+        $addressInput = $('.field-address input'),
+        $mapGeojsonTextarea = $('.django-leaflet-raw-textarea'),
+        $oldLat,
+        $oldLng,
+        $coordsUrl = $('#loci-geocode-url').attr('data-url'),
+        $addrUrl = $('#loci-reverse-geocode-url').attr('data-url');
 
     // define dummy gettext if django i18n is not enabled
     if (!gettext) { window.gettext = function (text) { return text; }; }
@@ -370,6 +376,138 @@ django.jQuery(function ($) {
         };
     }
 
+    // returns marker or featureGroup
+    function getMarkerFeatureGroup(option) {
+        var map = getMap(),
+            layer;
+        map.eachLayer(function (lay) {
+            if (lay.hasOwnProperty(option)) {
+                layer = lay;
+            }
+        });
+        return layer;
+    }
+    // returns placed marker
+    function getMarker() {
+        return getMarkerFeatureGroup('_latlng');
+    }
+
+    // returns map's feature group
+    function getFeatureGroup() {
+        return getMarkerFeatureGroup('_layers');
+    }
+
+    // update lat and lng
+    function updateLatLng(latlng) {
+        $oldLat = latlng.lat.toString();
+        $oldLng = latlng.lng.toString();
+    }
+
+    // update map view
+    function updateMapView(data) {
+        var geojson = '{ "type": "Point", "coordinates": [ ' +
+                        data.lng + ', ' + data.lat + '] }';
+        $mapGeojsonTextarea.val(geojson);
+        getMap().setView([data.lat, data.lng], 15);
+
+    }
+
+    // update map
+    function updateMap() {
+        var addressValue = $addressInput.val(),
+            message;
+        if (!addressValue) {
+            getFeatureGroup().clearLayers();
+            return;
+        }
+        $.get($coordsUrl, {address: addressValue})
+            .done(function (data) {
+                var marker = getMarker(),
+                    featureGroup = getFeatureGroup();
+                if (marker === undefined) {
+                    updateLatLng(data);
+                    featureGroup.addLayer(L.marker([data.lat, data.lng]));
+                } else {
+                    var latlng = marker.getLatLng();
+                    if (latlng.lat !== data.lat || latlng.lng !== data.lng) {
+                        message = gettext('The address was changed, would you like to ' +
+                                          'automatically update the location on the map?');
+                        if (confirm(message)) {
+                            updateLatLng(data);
+                            featureGroup.removeLayer(marker);
+                            featureGroup.addLayer(L.marker([data.lat, data.lng]));
+
+                        }
+                    }
+                }
+            })
+            .fail(function () {
+                message = gettext('Location with address: ' + $addressInput.val() + 'was not found.');
+                alert(message);
+            });
+    }
+
+    function updateAdress() {
+        var marker = getMarker(),
+            message,
+            latlng;
+        if (marker === undefined) {
+            return;
+        }
+        latlng = marker.getLatLng();
+        if (latlng.lat.toString() === $oldLat && latlng.lng.toString() === $oldLng) {
+            return;
+        }
+        updateLatLng(latlng);
+        $.get($addrUrl, {lat: latlng.lat, lng: latlng.lng})
+            .done(function (data) {
+                if (!$addressInput.val()) {
+                    $addressInput.val(data.address);
+                } else {
+                    message = gettext('The location on the map was changed, would you ' +
+                                      'like to update the address to');
+                    message += ' "' + data.address + '"?';
+                    if (confirm(message)) {
+                        $addressInput.val(data.address);
+                    }
+                }
+            })
+            .fail(function () {
+                message = gettext('Could not find any address related to this location.');
+                alert(message);
+            });
+    }
+
+    // triggers update of the address when the location on the map is changed
+    function updateAddressOnMapChange() {
+        var marker = getMarker();
+        if (!marker) { return; }
+        getMap().on('draw:edited', function (e) {
+            updateAdress();
+            updateMapView(marker.getLatLng());
+        });
+    }
+
+    $addressInput.change(function () {
+        updateMap();
+    });
+
+    $(window).on('load', function () {
+        var featureGroup = getFeatureGroup(),
+            marker = getMarker();
+        featureGroup.on('layeradd', function () {
+            updateAdress();
+            updateAddressOnMapChange();
+            marker = getMarker();
+            if (!marker) { return; }
+            updateMapView(marker.getLatLng());
+        });
+        if (marker !== undefined) {
+            updateLatLng(marker.getLatLng());
+            updateAddressOnMapChange();
+        }
+    });
+
     // show existing location
     var pk;
     if ($location.val()) {
